/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

public class Complexos
{
    /* METODOS DISPONIVEIS
        + Complexos()
        + Complexos(double novoReal, double novoImaginario)
        + void set(double novoReal, double novoImaginario)
        + void setReal(double novoReal)
        + double getReal()
        + double getImaginaria()
        + Complexos soma(Complexos cmp)
        + Complexos mult(Complexos cmp)
        + Complexos recebe(Complexos cmp)
        + boolean igual(Complexos cmp)
    */ 
    
    // Armazenam as partes real e imaginria do numero (a + bi)
    private double parteReal;
    private double parteImaginaria;
    
    // Construtor padro de um nmero complexo
    public Complexos()
    {
		parteReal = 0.0;
        parteImaginaria = 0.0;
    }
    
	// Construtor a partir de parmetros reais e imaginrios pr-definidos
	public Complexos(double novoReal, double novoImaginario)
    {
        parteReal = novoReal;
        parteImaginaria = novoImaginario;
    }

    // Seta um novo valor para um dado numero complexo
    public void set(double novoReal, double novoImaginario)
    {
		parteReal = novoReal;
        parteImaginaria = novoImaginario;
    }
    
    // Seta apenas a parte real do numero
    public void setReal(double novoReal)
    {
        parteReal = novoReal;
    }
    
    // Seta apenas a parte imaginaria do numero
    public void setImaginaria(double novoImaginario)
    {
        parteImaginaria = novoImaginario;
    }
 
    // Retorna a parte real
    public double getReal()
    {
        return parteReal;
    }

    // Retorna a parte imaginaria
    public double getImaginaria()
    {
        return parteImaginaria;
    }
    
    // Sobrecarga do operador de adicao
    public Complexos soma(Complexos cmp)
    {
        double novoReal, novoImaginario;
        novoReal = getReal() + cmp.getReal();
        novoImaginario = getImaginaria() + cmp.getImaginaria();
        Complexos tmp = new Complexos(novoReal, novoImaginario);
        
        return tmp;
    }
    
    // Sobrecarga do operador de multiplicao
    public Complexos mult(Complexos cmp)
    {
        double novoReal, novoImaginario;
        novoReal = getReal() * cmp.getReal() - getImaginaria() * cmp.getImaginaria();
        novoImaginario = getReal() * cmp.getImaginaria() + getImaginaria() * cmp.getReal();
        Complexos tmp = new Complexos(novoReal, novoImaginario);        
        return tmp;
    }
    
    // Sobrecarga do operador de atribuio
    public Complexos recebe(Complexos cmp)
    {
        if (cmp != this)
            set(cmp.getReal(), cmp.getImaginaria());
        return this;
    }
    
    // Sobrecarga do operador de igualdade
    public boolean igual(Complexos cmp)
    {
        if (Math.abs(cmp.getReal() - getReal()) >= Math.pow(10, -15))
            return false;
        if (Math.abs(cmp.getImaginaria() - getImaginaria()) >= Math.pow(10, -15))
            return false;

		return true;
    }
}
