/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

public class FrameDeutsch extends JInternalFrame 
{
	protected static final int	xOffset = 30, yOffset = 30;
	protected static int		contadorDeInstancias = 0;
	
	protected Container			c;
	protected Font		        fonteArial10;
	protected String		    opcoesCombo[];
	protected ImageIcon			imgTopo;
    protected JLabel		    labelTopo;
    protected JLabel            labelF_0;
    protected JLabel            labelF_1;
    protected JLabel            labelBloch_1;
    protected JLabel            labelBloch_2;
    protected ImageIcon         imgBloch_1;
    protected ImageIcon         imgBloch_2;
    protected ImageIcon         imgBloch_3;
    protected JLabel            labelResultado;
    protected JPanel            painelFuncoes;
    protected JPanel            painelQuBits;
    protected JPanel            painelRelatorios;
    protected JPanel            painelBotoes;
    protected JPanel            painelResultados;
    protected JPanel            painelF_0; 
    protected JPanel            painelF_1;
    protected JButton           btFechar;
    protected JButton           btProceder;
    protected JRadioButton      radioF0[];
    protected JRadioButton      radioF1[];
    protected JRadioButton      f0_0;
    protected JRadioButton      f0_1;
    protected JRadioButton      f1_0;
    protected JRadioButton      f1_1;
    protected ButtonGroup       grupoF0;
    protected ButtonGroup       grupoF1;
    protected HandlerRadio      handler;

    protected int               f0, f1;
    protected double            Uf;

	public FrameDeutsch() 
	{
        super("Simulao do Problema de DEUTSCH #" + (++contadorDeInstancias), 
              false,	// manipulvel
              true,		// fechvel
              false,	// maximizvel
              true);	// iconificvel
		
		fonteArial10	 = new Font("Arial", Font.PLAIN, 10);
        painelFuncoes    = new JPanel();
        painelQuBits     = new JPanel();
        painelRelatorios = new JPanel();
        painelBotoes     = new JPanel();
        painelResultados = new JPanel();
        painelF_0        = new JPanel();
        painelF_1        = new JPanel();
        btFechar         = new JButton("Fechar");
        btProceder       = new JButton("Proceder");
        grupoF0          = new ButtonGroup();
        grupoF1          = new ButtonGroup();
        f0_0             = new JRadioButton("F(0) = 0", true);
        f0_1             = new JRadioButton("F(0) = 1", false);
        f1_0             = new JRadioButton("F(1) = 0", true);
        f1_1             = new JRadioButton("F(1) = 1", false);
        handler          = new HandlerRadio();
        labelF_0         = new JLabel("Escolha F(0)");
        labelF_1         = new JLabel("Escolha F(1)");
        labelResultado   = new JLabel("");
        imgBloch_1      = new ImageIcon(this.getClass().getResource("pics/esfera_1.gif"));
        imgBloch_2      = new ImageIcon(this.getClass().getResource("pics/esfera_2.gif"));
        imgBloch_3      = new ImageIcon(this.getClass().getResource("pics/esfera_3.gif"));
        labelBloch_1    = new JLabel(imgBloch_1);
        labelBloch_2    = new JLabel(imgBloch_1);

        grupoF0.add(f0_0);
        grupoF0.add(f0_1);
        grupoF1.add(f1_0);
        grupoF1.add(f1_1);
        
        f0_0.addItemListener(handler);
        f0_1.addItemListener(handler);
        f1_0.addItemListener(handler);
        f1_1.addItemListener(handler);

		Border espaco = BorderFactory.createEmptyBorder(5,5,5,5);
        painelFuncoes.setBorder(BorderFactory.createCompoundBorder
   	    					  (criarBorda("FUNES BINRIAS"), espaco));
        painelQuBits.setBorder(BorderFactory.createCompoundBorder
   	    					  (criarBorda("ACOMPANHAMENTO"), espaco));
        painelRelatorios.setBorder(BorderFactory.createCompoundBorder
   	    					  (criarBorda("DIAGRAMAS DE BLOCH"), espaco));
        painelBotoes.setBorder(BorderFactory.createCompoundBorder
   	    					  (criarBorda("OPES RPIDAS"), espaco));
        painelResultados.setBorder(BorderFactory.createCompoundBorder
   	    					  (criarBorda("RESULTADOS"), espaco));

        painelFuncoes.setLayout(new GridLayout(4,1));
        painelQuBits.setLayout(new GridLayout(1,1));
        painelRelatorios.setLayout(new GridLayout(1,2));
        painelBotoes.setLayout(new GridLayout(1,2));
        painelResultados.setLayout(new GridLayout(1,1));
        painelF_0.setLayout(new GridLayout(1,2));
        painelF_1.setLayout(new GridLayout(1,2));

        f0_0.setFont(fonteArial10);
        f0_1.setFont(fonteArial10);
        f1_0.setFont(fonteArial10);
        f1_1.setFont(fonteArial10);
        btFechar.setFont(fonteArial10);
        btProceder.setFont(fonteArial10);
        labelF_0.setFont(fonteArial10);
        labelF_1.setFont(fonteArial10);
        labelResultado.setFont(fonteArial10);

        btFechar.setMnemonic('F');
        btProceder.setMnemonic('P');
        btFechar.setToolTipText("Fecha este quadro do Problema de DEUTSCH");
        btProceder.setToolTipText("Verifica se F(0) e F(1) so CONSTANTES ou BALANCEADAS");

        setSize(640,480);
        setLocation(xOffset*contadorDeInstancias, yOffset*contadorDeInstancias);
		setOpaque(false);
		
		imgTopo			= new ImageIcon(this.getClass().getResource("pics/deutschLogo.png"));
		labelTopo		= new JLabel(imgTopo);

		// Setando o posicionamento dos itens na tela
		labelTopo.setBounds		    (100,  10, 400, 40);
    	painelFuncoes.setBounds     ( 10,  50, 145, 125);
        painelQuBits.setBounds      ( 10, 180, 145, 260);
        painelRelatorios.setBounds  (160,  50, 465, 325);
        painelBotoes.setBounds      (390, 380, 235, 60);
        painelResultados.setBounds  (160, 380, 225, 60);

		// Setando o boto padro
		getRootPane().setDefaultButton(btFechar);

        setBackground(Color.white);
        painelFuncoes.setBackground(Color.white);
        painelQuBits.setBackground(Color.white);
        painelRelatorios.setBackground(Color.white);
        painelBotoes.setBackground(Color.white);
        painelResultados.setBackground(Color.white);
        painelF_0.setBackground(Color.white);
        painelF_1.setBackground(Color.white);
        f0_0.setBackground(Color.white);
        f0_1.setBackground(Color.white);
        f1_0.setBackground(Color.white);
        f1_1.setBackground(Color.white);

        painelResultados.add(labelResultado);
        painelF_0.add(f0_0);
        painelF_0.add(f0_1);
        painelF_1.add(f1_0);
        painelF_1.add(f1_1);
        painelBotoes.add(btProceder);
        painelBotoes.add(btFechar);
        painelFuncoes.add(labelF_0);
        painelFuncoes.add(painelF_0);
        painelFuncoes.add(labelF_1);
        painelFuncoes.add(painelF_1);
        painelRelatorios.add(labelBloch_1);
        painelRelatorios.add(labelBloch_2);

        c = getContentPane();
		c.setLayout(null);
		c.add(labelTopo);
        c.add(painelFuncoes);
        c.add(painelQuBits);
        c.add(painelRelatorios);
        c.add(painelBotoes);
        c.add(painelResultados);

   		btFechar.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) { dispose(); }
		});

        btProceder.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) { resolverProblemaDeutsch(); }
		});
	}

	private Border criarBorda(String titulo)
	{
		Border bd = BorderFactory.createTitledBorder(
					BorderFactory.createEtchedBorder(),
					titulo, TitledBorder.LEFT, 
					TitledBorder.TOP, fonteArial10);

		return bd;
	}

    private void resolverProblemaDeutsch()
    {
        double quBit1, quBit2;

        quBit1 = Math.pow(-1, f0);
        quBit2 = Math.pow(-1, f1);

        Uf = quBit1 + quBit2; 

        if ( Uf == 2)
        {
            labelResultado.setText("F(0) = " + f0 + " e F(1) = " + f1 + " so CONSTANTES");
            labelBloch_1.setIcon(imgBloch_2);
            labelBloch_2.setIcon(imgBloch_2);
        }
        else if (Uf == 0)
        {
            labelResultado.setText("F(0) = " + f0 + " e F(1) = " + f1 + " so BALANCEADAS");
            labelBloch_1.setIcon(imgBloch_2);
            labelBloch_2.setIcon(imgBloch_3);
        }
        else
        {
            labelResultado.setText("F(0) = " + f0 + " e F(1) = " + f1 + " so CONSTANTES");
            labelBloch_1.setIcon(imgBloch_3);
            labelBloch_2.setIcon(imgBloch_3);
        }
    }

    private class HandlerRadio implements ItemListener
    {
        public void itemStateChanged(ItemEvent e)
        {
            if(e.getSource() == f0_0)
                f0 = 0;
            else if(e.getSource() == f0_1)
                f0 = 1;
            else if(e.getSource() == f1_0)
                f1 = 0;
            else if(e.getSource() == f1_1)
                f1 = 1;
        }
    }
}
