/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

public class FrameShor extends JInternalFrame 
{
	protected static final int	xOffset = 30, yOffset = 30;
	protected static int		contadorDeInstancias = 0;
	
	protected Container			c;
	protected Font		        fonteArial10;
	protected String		    opcoesCombo[];
	protected ShorTask			tarefa;
    protected Timer				timer;
	protected ImageIcon			imgTopo;
	protected JProgressBar		status;
	protected JTextField	    campoNumero;
	protected JScrollPane		scrollPane[];
	protected JTextArea			areaTexto[];
	protected JLabel		    labelTopo;
	protected JButton			btFatorar;
	protected JButton			btPausar;
	protected JButton			btFechar;
	protected JComboBox			comboEstados;
	protected JPanel			painel[];

	public FrameShor() 
	{
        super("Algoritmo de Fatoracao de PETER SHOR #" + (++contadorDeInstancias), 
              false,	// manipulvel
              true,		// fechvel
              false,	// maximizvel
              true);	// iconificvel
		
		setOpcoesCombo();
		fonteArial10	= new Font("Arial", Font.PLAIN, 10);
		
		setSize(640,480);
        setLocation(xOffset*contadorDeInstancias, yOffset*contadorDeInstancias);
		setOpaque(false);
		setBackground(Color.white);

		imgTopo			= new ImageIcon(this.getClass().getResource("pics/shorLogo.png"));
		tarefa			= new ShorTask();
		status			= new JProgressBar(0, tarefa.getTamanhoTarefa());
		labelTopo		= new JLabel(imgTopo);
		campoNumero		= new JTextField(10);
		btFatorar		= new JButton("Fatorar");
		btPausar		= new JButton("Pausar");
		btFechar		= new JButton("Fechar");
		comboEstados	= new JComboBox(opcoesCombo);
		scrollPane		= new JScrollPane[6];
		areaTexto		= new JTextArea[6];
		areaTexto[0]    = new JTextArea("", 45, 100);
		areaTexto[1]	= new JTextArea("", 45, 100);
		areaTexto[2]	= new JTextArea("", 45, 100);
		areaTexto[3]	= new JTextArea("", 45, 100);
		areaTexto[4]	= new JTextArea("", 45, 100);
		areaTexto[5]	= new JTextArea("", 45, 100);
		scrollPane[0]	= new JScrollPane(areaTexto[0]);
		scrollPane[1]	= new JScrollPane(areaTexto[1]);
		scrollPane[2]	= new JScrollPane(areaTexto[2]);
		scrollPane[3]	= new JScrollPane(areaTexto[3]);
		scrollPane[4]	= new JScrollPane(areaTexto[4]);	
		scrollPane[5]	= new JScrollPane(areaTexto[5]);

		// Setando as fontes 
		for (int i = 0; i < areaTexto.length; i++)
		{
			areaTexto[i].setFont(fonteArial10);
			areaTexto[i].setEditable(false);
		}

		// Setando o posicionamento dos itens na tela
		labelTopo.setBounds		(100, 0, 400, 40);

		for (int i = 0; i < scrollPane.length; i++)
		{
			scrollPane[i].setBorder(BorderFactory.createEmptyBorder());
			scrollPane[i].setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			scrollPane[i].setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		}

		campoNumero.setBorder(BorderFactory.createEmptyBorder());
				
		// Setando o boto padro
		getRootPane().setDefaultButton(btFatorar);

		Component cmpReg1[]		= {scrollPane[0]};
		Component cmpReg2[]		= {scrollPane[1]};
		Component cmpProb1[]	= {scrollPane[2]};
		Component cmpProb2[]	= {scrollPane[3]};
		Component cmpMod[]		= {scrollPane[4]};
		Component cmpTracer[]	= {scrollPane[5]};
		Component cmpBotoes[]	= {btFatorar, btPausar, btFechar};
		Component cmpStatus[]	= {status};
		Component cmpNumero[]	= {campoNumero};
		Component cmpRelatorio[]= {comboEstados};

		painel = new JPanel[10];
		painel[0] = criarPainel("STATUS DO REGISTRADOR QUNTICO #1", cmpReg1, 350, 40, 275, 290);
		painel[1] = criarPainel("STATUS DO REGISTRADOR QUNTICO #2", cmpReg2, 350, 40, 275, 290);
		painel[2] = criarPainel("PROBABILIDADES DE AMPLITUDE #1", cmpProb1, 350, 40, 275, 290);
		painel[3] = criarPainel("PROBABILIDADES DE AMPLITUDE #2", cmpProb2, 350, 40, 275, 290);
		painel[4] = criarPainel("MDULOS (X^q mod N) CALCULADOS", cmpMod, 350, 40, 275, 290);
		painel[5] = criarPainel("OPES RPIDAS", cmpBotoes, 350, 390, 275, 50);
		painel[6] = criarPainel("PROGRESSO DA EXECUO DO ALGORITMO", cmpStatus, 350, 335, 275, 55);
		painel[7] = criarPainel("ACOMPANHAMENTO DA EXECUO", cmpTracer, 10, 100, 335, 340);
		painel[8] = criarPainel("NMERO A SER FATORADO", cmpNumero, 10, 40, 165, 50);
		painel[9] = criarPainel("OUTROS RELATRIOS", cmpRelatorio, 180, 40, 165, 50);

		c = getContentPane();
		c.setLayout(null);
		c.add(labelTopo);

		for (int i = 0; i < painel.length; i++)
			c.add(painel[i]);
		
		for (int i = 1; i < 5; i++)
			painel[i].setVisible(false);

		// Setando a fonte dos botoes
		btFechar.setFont(fonteArial10);
		btFatorar.setFont(fonteArial10);
		btPausar.setFont(fonteArial10);
		campoNumero.setFont(fonteArial10);
		comboEstados.setFont(fonteArial10);

		// Setando maximo de opes visveis
		comboEstados.setMaximumRowCount(4);

		// Setando habilitao
		btPausar.setEnabled(false);
		btFatorar.setEnabled(false);

		status.setValue(0);
        status.setStringPainted(true);

		btFatorar.addActionListener   (new HandlerBotoes(FrameShor.this, this));
		btPausar.addActionListener	  (new HandlerBotoes(FrameShor.this, this));
		btFechar.addActionListener	  (new HandlerBotoes(FrameShor.this, this));
		campoNumero.addActionListener (new HandlerBotoes(FrameShor.this, this));
		comboEstados.addActionListener(new HandlerBotoes(FrameShor.this, this));
		
		// Setando tooltips e mnemonics
		btFatorar.setMnemonic('F');
		btFechar.setMnemonic('F');
		btPausar.setMnemonic('P');
		btFatorar.setToolTipText("Inicia a fatorao do nmero fornecido");
		btFechar.setToolTipText("Fecha este quadro do Algoritmo de Shor");
		btPausar.setToolTipText("Pausa a execuo da fatorao");
		
		timer = criarTimer();
	}

	private void setOpcoesCombo()
	{
		opcoesCombo = new String[5];
		opcoesCombo[0] = "Registrador #1";
		opcoesCombo[1] = "Registrador #2";
		opcoesCombo[2] = "Probabilidades #1";
		opcoesCombo[3] = "Probabilidades #2";
		opcoesCombo[4] = "X^q MOD N";
	}

	private Timer criarTimer()
	{
		Timer t = new Timer(SAQua.UM_SEGUNDO, new ActionListener() 
		{
            public void actionPerformed(ActionEvent evt)
			{
				status.setValue(tarefa.getStatus());
				areaTexto[5].setText(tarefa.getMensagem());
				areaTexto[0].setText(tarefa.getEstadosObjeto(1));
				areaTexto[1].setText(tarefa.getEstadosObjeto(2));

				// DESABILITADOR POR QUESTES DE PERFORMANCE!!!
				//areaTexto[2].setText(tarefa.getEstadosObjeto(3));
				//areaTexto[3].setText(tarefa.getEstadosObjeto(4));
				//areaTexto[4].setText(tarefa.getEstadosObjeto(5));

				if (tarefa.done()) 
				{
					Toolkit.getDefaultToolkit().beep();
					timer.stop();
					campoNumero.setEnabled(true);
					btFatorar.setEnabled(false);
					btPausar.setEnabled(false);
					status.setValue(status.getMinimum());
					areaTexto[5].setText(tarefa.getMensagem());
					campoNumero.setText("");
				}
			}
		});

		return t;
	}
	
	private JPanel criarPainel(String titulo, Component cmps[], 
							   int x, int y, int tx, int ty)
	{
		JPanel painel = new JPanel();

		painel.setLayout(new GridLayout(1, cmps.length));
		painel.setBackground(Color.white);
		
		for (int i = 0; i < cmps.length; i++)
			painel.add(cmps[i]);

		Border espaco = BorderFactory.createEmptyBorder(2,2,2,2);

		Border bd = BorderFactory.createTitledBorder(
					BorderFactory.createEtchedBorder(),
					titulo, TitledBorder.LEFT, 
					TitledBorder.TOP, fonteArial10);

		painel.setBorder(BorderFactory.createCompoundBorder
							  (bd, espaco));

		painel.setBounds(x, y, tx, ty);

		return painel;
	}
}